/*
 * ll_dmadrivenusartcore.cpp
 *
 *  Created on: Nov 4, 2019
 *      Author: abody
 */

#include <f1ll/usart_core.h>

namespace f1ll {

usart_core::usart_core(USART_TypeDef *usart, DMA_TypeDef *dma,
                       uint32_t channelRx, uint32_t channelTx)
    : m_usart(usart), m_rxDma(dma, channelRx), m_txDma(dma, channelTx) {
  uint32_t status = usart->SR;
  volatile uint32_t tmpreg =
      usart->DR; // clearing some of the error/status bits in the USART
  (void)tmpreg;
  (void)status;

  *m_txDma.get_ifc_reg() = m_txDma.get_gi_mask();
  *m_rxDma.get_ifc_reg() = m_rxDma.get_gi_mask();

  LL_DMA_EnableIT_HT(dma, channelRx);
  LL_DMA_EnableIT_TC(dma, channelRx);
  LL_DMA_EnableIT_TE(dma, channelRx);
  LL_DMA_EnableIT_HT(dma, channelTx);
  LL_DMA_EnableIT_TC(dma, channelTx);
  LL_DMA_EnableIT_TE(dma, channelTx);
}

void usart_core::usart_isr() {
  uint32_t status = m_usart->SR;
  volatile uint32_t tmpreg =
      m_usart->DR; // clearing some of the error/status bits in the HW
  (void)tmpreg;

  if (LL_USART_IsEnabledIT_TC(m_usart) &&
      LL_USART_IsActiveFlag_TC(m_usart)) { // transmission complete
    LL_USART_DisableIT_TC(m_usart);
    transmission_complete();
  }
  if (LL_USART_IsEnabledIT_IDLE(m_usart) && (status & USART_SR_IDLE)) {
    receiver_idle();
  }
  if (LL_USART_IsEnabledIT_ERROR(m_usart)) {
    if (status & USART_SR_FE) {
      framing_error();
    }
    if (status & USART_SR_ORE) {
      overrun();
    }
  }
}

void usart_core::rx_dma_isr() {
  if (*m_rxDma.get_is_reg() & m_rxDma.get_tc_mask()) {
    *m_rxDma.get_ifc_reg() = m_rxDma.get_tc_mask();
    if (m_rxDma.is_enabled_it_tc()) {
      rx_dma_transfer_complete();
    }
  }
  if (*m_rxDma.get_is_reg() & m_rxDma.get_ht_mask()) {
    *m_rxDma.get_ifc_reg() = m_rxDma.get_ht_mask();
    if (m_rxDma.is_enabled_it_ht()) {
      rx_dma_half_transfer();
    }
  }
  if (*m_rxDma.get_is_reg() & m_rxDma.get_te_mask()) {
    *m_rxDma.get_ifc_reg() = m_rxDma.get_te_mask();
    if (m_rxDma.is_enabled_it_te()) {
      rx_dma_error();
    }
  }
}

void usart_core::tx_dma_isr() {
  if (*m_txDma.get_is_reg() & m_txDma.get_tc_mask()) { // DMA transfer complete
    *m_txDma.get_ifc_reg() = m_txDma.get_tc_mask();
    if (m_txDma.is_enabled_it_tc()) {
      tx_dma_transfer_complete();
    }
  }
  if (*m_txDma.get_is_reg() & m_txDma.get_ht_mask()) {
    *m_txDma.get_ifc_reg() = m_txDma.get_ht_mask();
    if (m_txDma.is_enabled_it_ht()) {
      tx_dma_half_transfer();
    }
  }
  if (*m_txDma.get_is_reg() & m_txDma.get_te_mask()) {
    *m_txDma.get_ifc_reg() = m_txDma.get_te_mask();
    if (m_txDma.is_enabled_it_te()) {
      tx_dma_error();
    }
  }
}

void usart_core::setup_transmit(void const *buffer, uint16_t length) {
  LL_DMA_ConfigAddresses(m_txDma.get_dma(), m_txDma.get_channel(),
                         reinterpret_cast<uint32_t>(buffer),
                         LL_USART_DMA_GetRegAddr(m_usart),
                         LL_DMA_DIRECTION_MEMORY_TO_PERIPH);
  LL_DMA_SetDataLength(m_txDma.get_dma(), m_txDma.get_channel(), length);
  LL_USART_EnableDMAReq_TX(m_usart);
  LL_DMA_EnableChannel(m_txDma.get_dma(), m_txDma.get_channel());
}

void usart_core::setup_receive(void *buffer, uint16_t length) {
  LL_DMA_ConfigAddresses(m_rxDma.get_dma(), m_rxDma.get_channel(),
                         LL_USART_DMA_GetRegAddr(m_usart),
                         reinterpret_cast<uint32_t>(buffer),
                         LL_DMA_DIRECTION_PERIPH_TO_MEMORY);
  LL_DMA_SetDataLength(m_rxDma.get_dma(), m_rxDma.get_channel(), length);
  LL_USART_EnableDMAReq_RX(m_usart);
  LL_USART_ClearFlag_ORE(m_usart);
  LL_DMA_EnableChannel(m_rxDma.get_dma(), m_rxDma.get_channel());
}

} // namespace f1ll
